import logging
from typing import List, Union, Optional

from ldap.ldapobject import SimpleLDAPObject as LdapObject

from . import _groups
from ._object_classes import Computer, Group, create_records
from ._query import object_exists, get, modify, move, ldap_escape, format_distinguished_name

logger = logging.getLogger(__name__)
class_plural = "computers"
object_class = "computer"


@create_records(Computer)
def _get(
    connection: LdapObject, search_base: Optional[str] = None, search_scope: Optional[int] = None, **query_params
) -> List[Computer]:
    return get(
        f"(objectClass={object_class})", connection, search_base=search_base, search_scope=search_scope, **query_params
    )


@object_exists(_get, class_plural)
def _group_memberships(connection: LdapObject, distinguished_name: str) -> List[Group]:
    return _groups._get(connection, member=format_distinguished_name(distinguished_name),)


@object_exists(_get, class_plural)
def _modify(
    connection: LdapObject, distinguished_name: str, key: str, value: Union[int, str, bytes], operation: str
) -> None:
    return modify(
        connection,
        # Un-Escape backslashes in distinguished names when modifying objects.
        ldap_escape(distinguished_name.replace("\\\\", "\\"), is_distinguished_name=True),
        key,
        value,
        operation,
    )


@object_exists(_get, class_plural)
def _move(connection: LdapObject, distinguished_name: str, new_parent: str) -> None:
    return move(connection, distinguished_name, new_parent)
